<?php

namespace Drupal\core_views_facets\Plugin\facets\facet_source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\core_views_facets\CoreViewsFacetsContextualFilterTypeManager;
use Drupal\views\Plugin\views\argument\ArgumentPluginBase;
use Drupal\views\Views;

/**
 * Represents a facet source of the core views with contextual filters.
 *
 * @FacetsFacetSource(
 *   id = "core_views_contextual_filter",
 *   deriver = "Drupal\core_views_facets\Plugin\facets\facet_source\CoreViewsContextualFilterDeriver"
 * )
 */
class CoreViewsContextualFilter extends CoreViewsFacetSourceBase {

  /**
   * The filter type plugin manager.
   *
   * @var CoreViewsFacetsContextualFilterTypeManager
   */
  protected $contextualFilterTypePluginManager;

  /**
   * {@inheritdoc}
   */
  public function isRenderedInCurrentRequest() {
    if (parent::isRenderedInCurrentRequest()) {
      return TRUE;
    }
    elseif (drupal_static('core_views_contextual_filter_ajax_rendered_status')) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, CoreViewsFacetsContextualFilterTypeManager $contextual_filter_type_plugin_manager, array $configuration, $plugin_id, array $plugin_definition, $query_type_plugin_manager) {
    parent::__construct($entity_type_manager, $configuration, $plugin_id, $plugin_definition, $query_type_plugin_manager);

    $this->contextualFilterTypePluginManager = $contextual_filter_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\facets\QueryType\QueryTypePluginManager $query_type_plugin_manager */
    $query_type_plugin_manager = $container->get('plugin.manager.facets.query_type');

    /** @var EntityTypeManagerInterface $entity_type_manager */
    $entity_type_manager = $container->get('entity_type.manager');

    /** @var CoreViewsFacetsContextualFilterTypeManager $contextual_filter_type_plugin_manager */
    $contextual_filter_type_plugin_manager = $container->get('plugin.manager.core_views_facets.contextual_filter_types');

    /** @var \Drupal\search_api\Query\ResultsCacheInterface $results_cache */
    return new static($entity_type_manager, $contextual_filter_type_plugin_manager, $configuration, $plugin_id, $plugin_definition, $query_type_plugin_manager);
  }

  /**
   * {@inheritdoc}
   */
  public function fillFacetsWithResults($facets) {
    foreach ($facets as $facet_id => $facet) {
      if ($facet->getOnlyVisibleWhenFacetSourceIsVisible()) {
        // Ignore currently unnecessary facets.
        $facet_source = $facet->getFacetSource();
        if (!$facet_source->isRenderedInCurrentRequest()) {
          continue;
        }
      }

      $view = Views::getView($this->pluginDefinition['view_id']);
      $view->setDisplay($this->pluginDefinition['view_display']);

      $views_arguments = $this->getFields();
      reset($views_arguments);
      if (!empty($views_arguments[$facet->getFieldIdentifier()])) {
        $facet_argument = $views_arguments[$facet->getFieldIdentifier()];
      }
      else {
        return;
      }

      $request_arguments = [];
      $map = $this->getViewsArgumentsMap();

      foreach ($map as $current_argument) {
        //if ($facet->getFieldIdentifier() == $current_argument['views_argument_id']) {
        //  $request_arguments[] = $current_argument['neutral_value'];
       // }
        //else {
          if ($current_argument['active']) {
            $request_arguments[] = $current_argument['value'];
          }
          else {
            $request_arguments[] = $current_argument['neutral_value'];
          }
        //}
      }

      $view->setArguments($request_arguments);
      $view->build($this->pluginDefinition['view_display']);

      $facet_core_views_contextual_filter_plugin = $this->loadFacetCoreViewsContextualFilterTypePlugin($facet_argument);
      if (empty($facet_core_views_contextual_filter_plugin)) {
        return;
      }

      $query = $facet_core_views_contextual_filter_plugin->prepareQuery($view, $facet_argument, $facet);
      if (empty($query)) {
        return;
      }

      $rows = $query->execute();

      $facet_results = [];
      while ($row = $rows->fetchObject()) {
        $facet_results[] = $facet_core_views_contextual_filter_plugin->processDatabaseRow($row, $facet_argument, $facet);
      }

      $configuration = array(
        'query' => NULL,
        'facet' => $facet,
      );

      $facet->setResults($facet_results);

      // Get the Facet Specific Query Type so we can process the results
      // using the build() function of the query type.
      $query_type = $this->queryTypePluginManager->createInstance($facet->getQueryType(), $configuration);
      $query_type->build();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $arguments = $this->getFields();

    /* @var ArgumentPluginBase $argument */
    foreach ($arguments as $argument_id => $argument) {
      $form['field_identifier']['#options'][$argument_id] = $argument->adminLabel();
    }

    $facet_core_views_contextual_filter_plugin = $this->loadFacetCoreViewsContextualFilterTypePlugin($argument);
    if (!empty($facet_core_views_contextual_filter_plugin)) {
      $form = $facet_core_views_contextual_filter_plugin->buildConfigurationForm($form, $form_state, $this->facet, $this, $argument);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $facet_source_id = $this->facet->getFacetSourceId();
    $views_arguments = $this->getFields();

    $field_identifier = $form_state->getValue('facet_source_configs')[$facet_source_id]['field_identifier'];

    if (empty($field_identifier) || empty($views_arguments[$field_identifier])) {
      return;
    }
    $this->facet->setFieldIdentifier($field_identifier);

    $argument = $views_arguments[$field_identifier];

    $facet_core_views_contextual_filter_plugin = $this->loadFacetCoreViewsContextualFilterTypePlugin($argument);
    if (!empty($facet_core_views_contextual_filter_plugin)) {
      $data = $form_state->getValue('facet_source_configs')[$facet_source_id];
      $facet_core_views_contextual_filter_plugin->submitConfigurationForm($data, $this->facet, $this, $argument);
    }
  }

  /**
   * Retrieve the argument definition from the current view.
   *
   * @param string|null $argument_id
   *   The string ID of a argument.
   *
   * @return array|bool
   *   An array with the definition of the argument, or FALSE when the plugin
   *   doesn't exist.
   */
  public function getViewsArgumentDefinition($argument_id = NULL) {
    if (empty($argument_id) || empty($this->view)) {
      return FALSE;
    }

    /** @var ArgumentPluginBase $argument */
    $filter = $this->view->getHandler($this->pluginDefinition['view_display'], 'argument', $argument_id);
    return $filter ?: FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $arguments = [];
    foreach ($this->view->getHandlers('argument', $this->view->current_display) as $argument_id => $argument) {
      $arguments[$argument_id] = $this->view->display_handler->getHandler('argument', $argument_id);
    }

    return $arguments;
  }

  /**
   * Load the core_views_facet contextual filter type or fall back to generic.
   *
   * @param ArgumentPluginBase $argument
   *   The views filter handler.
   *
   * @return \Drupal\core_views_facets\CoreViewsFacetsContextualFilterTypeInterface
   *   The loaded filter type plugin or null.
   */
  protected function loadFacetCoreViewsContextualFilterTypePlugin(ArgumentPluginBase $argument) {
    $filter_type_definitions = $this->contextualFilterTypePluginManager->getDefinitions();

    $custom_filter_id = $argument->view->id() . "-" . $argument->field;
    // Allows to handle custom scenarios.
    if (!empty($filter_type_definitions[$custom_filter_id])) {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsContextualFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->contextualFilterTypePluginManager->createInstance($custom_filter_id);
    }
    // Default filter types for specific filter plugins.
    elseif (!empty($filter_type_definitions[$argument->pluginId])) {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsContextualFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->contextualFilterTypePluginManager->createInstance($argument->pluginId);
    }
    // Generic filter type.
    else {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsContextualFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->contextualFilterTypePluginManager->createInstance('generic');
    }

    return $facet_core_views_filter_plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $plugin_id_array = explode(':', $this->pluginId);
    list($view_id, ) = explode('__', $plugin_id_array[1]);
    $dependencies['config'] = ['views.view.' . $view_id];
    $dependencies['module'] = ['core_views_facets', 'views'];

    return $dependencies;
  }

}
