<?php

namespace Drupal\core_views_facets\Plugin\facets\facet_source;

use Drupal\Core\Database\DatabaseExceptionWrapper;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\core_views_facets\CoreViewsFacetsExposedFilterTypeManager;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\views\Views;
use Drupal\Core\Form\FormStateInterface;

/**
 * Represents a facet source of the core views with exposed filters.
 *
 * @FacetsFacetSource(
 *   id = "core_views_exposed_filter",
 *   deriver = "Drupal\core_views_facets\Plugin\facets\facet_source\CoreViewsExposedFilterDeriver"
 * )
 */
class CoreViewsExposedFilter extends CoreViewsFacetSourceBase {

  /**
   * The filter type plugin manager.
   *
   * @var CoreViewsFacetsExposedFilterTypeManager
   */
  protected $exposedFilterTypePluginManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, CoreViewsFacetsExposedFilterTypeManager $exposed_filter_type_plugin_manager, array $configuration, $plugin_id, array $plugin_definition, $query_type_plugin_manager) {
    parent::__construct($entity_type_manager, $configuration, $plugin_id, $plugin_definition, $query_type_plugin_manager);

    $this->exposedFilterTypePluginManager = $exposed_filter_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\facets\QueryType\QueryTypePluginManager $query_type_plugin_manager */
    $query_type_plugin_manager = $container->get('plugin.manager.facets.query_type');

    /** @var EntityTypeManagerInterface $entity_type_manager */
    $entity_type_manager = $container->get('entity_type.manager');

    /** @var CoreViewsFacetsExposedFilterTypeManager $exposed_filter_type_plugin_manager */
    $exposed_filter_type_plugin_manager = $container->get('plugin.manager.core_views_facets.exposed_filter_types');

    /** @var \Drupal\search_api\Query\ResultsCacheInterface $results_cache */
    return new static($entity_type_manager, $exposed_filter_type_plugin_manager, $configuration, $plugin_id, $plugin_definition, $query_type_plugin_manager);
  }

  /**
   * {@inheritdoc}
   */
  public function isRenderedInCurrentRequest() {
    if (parent::isRenderedInCurrentRequest()) {
      return TRUE;
    }
    elseif (drupal_static('core_views_exposed_filter_ajax_rendered_status')) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function fillFacetsWithResults($facets) {
    foreach ($facets as $facet_id => $facet) {
      if ($facet->getOnlyVisibleWhenFacetSourceIsVisible()) {
        // Ignore currently unnecessary facets.
        $facet_source = $facet->getFacetSource();
        if (!$facet_source->isRenderedInCurrentRequest()) {
          continue;
        }
      }

      $request_arguments = [];
      $map = $this->getViewsArgumentsMap();

      foreach ($map as $current_argument) {
        if ($current_argument['active']) {
          $request_arguments[] = $current_argument['value'];
        }
        else {
          break;
        }
      }

      $view = Views::getView($this->pluginDefinition['view_id']);
      $view->setArguments($request_arguments);
      $view->setDisplay($this->pluginDefinition['view_display']);

      $filters = $this->getFields();

      if (empty($filters[$facet->getFieldIdentifier()])) {
        return;
      }

      $filter = $filters[$facet->getFieldIdentifier()];

      $view->build($this->pluginDefinition['view_display']);

      $facet_core_views_exposed_filter_plugin = $this->loadFacetCoreViewsExposedFilterTypePlugin($filter);
      if (empty($facet_core_views_exposed_filter_plugin)) {
        return;
      }

      $query = $facet_core_views_exposed_filter_plugin->prepareQuery($view, $filter, $facet);
      if (empty($query)) {
        return;
      }

      try {
        $rows = $query->execute();
      }
      catch (DatabaseExceptionWrapper $e) {
        continue;
      }

      $facet_results = [];
      while ($row = $rows->fetchObject()) {
        $facet_results[] = $facet_core_views_exposed_filter_plugin->processDatabaseRow($row, $filter, $facet);
      }

      $configuration = array(
        'query' => NULL,
        'facet' => $facet,
      );

      $facet->setResults($facet_results);

      // Get the Facet Specific Query Type so we can process the results
      // using the build() function of the query type.
      $query_type = $this->queryTypePluginManager->createInstance($facet->getQueryType(), $configuration);
      $query_type->build();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $filters = $this->getFields();

    /* @var FilterPluginBase $filter */
    foreach ($filters as $filter_id => $filter) {
      if (!$filter->isExposed()) {
        continue;
      }
      $form['field_identifier']['#options'][$filter_id] = $filter->adminLabel();

      $facet_core_views_filter_plugin = $this->loadFacetCoreViewsExposedFilterTypePlugin($filter);
      if (!empty($facet_core_views_filter_plugin)) {
        $form = $facet_core_views_filter_plugin->buildConfigurationForm($form, $form_state, $this->facet, $this, $filter);
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $facet_source_id = $this->facet->getFacetSourceId();
    $views_filters = $this->getFields();

    $field_identifier = $form_state->getValue('facet_source_configs')[$facet_source_id]['field_identifier'];

    if (empty($field_identifier) || empty($views_filters[$field_identifier])) {
      return;
    }
    $this->facet->setFieldIdentifier($field_identifier);

    $filter = $views_filters[$field_identifier];

    $facet_core_views_filter_plugin = $this->loadFacetCoreViewsExposedFilterTypePlugin($filter);
    if (!empty($facet_core_views_filter_plugin)) {
      $data = $form_state->getValue('facet_source_configs')[$facet_source_id];
      $facet_core_views_filter_plugin->submitConfigurationForm($data, $this->facet, $this, $filter);
    }

  }

  /**
   * Retrieve the filter definition from the current view.
   *
   * @param string|null $filter_id
   *   The string ID of a filter.
   *
   * @return array|bool
   *   An array with the definition of the filter, or FALSE when the plugin
   *   doesn't exist.
   */
  public function getViewsFilterDefinition($filter_id = NULL) {
    if (empty($filter_id) || empty($this->view)) {
      return FALSE;
    }

    /** @var FilterPluginBase $filter */
    $filter = $this->view->getHandler($this->pluginDefinition['view_display'], 'filter', $filter_id);
    return $filter ?: FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $filters = [];
    foreach ($this->view->getHandlers('filter', $this->view->current_display) as $filter_id => $filter) {
      if (empty($filter['exposed']) || $filter['exposed'] != TRUE) {
        continue;
      }
      $filters[$filter_id] = $this->view->display_handler->getHandler('filter', $filter_id);
    }

    return $filters;
  }

  /**
   * Load the core_views_facet filter type or fall back to generic.
   *
   * @param FilterPluginBase $filter
   *   The views filter handler.
   *
   * @return \Drupal\core_views_facets\CoreViewsFacetsExposedFilterTypeInterface
   *   The loaded filter type plugin or null.
   */
  protected function loadFacetCoreViewsExposedFilterTypePlugin(FilterPluginBase $filter) {
    $filter_type_definitions = $this->exposedFilterTypePluginManager->getDefinitions();

    $custom_filter_id = $filter->view->id() . "-" . $filter->field;
    // Allows to handle custom scenarios.
    if (!empty($filter_type_definitions[$custom_filter_id])) {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsExposedFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->exposedFilterTypePluginManager->createInstance($custom_filter_id);
    }
    // Default filter types for specific filter plugins.
    elseif (!empty($filter_type_definitions[$filter->pluginId])) {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsExposedFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->exposedFilterTypePluginManager->createInstance($filter->pluginId);
    }
    // Generic filter type.
    else {
      /** @var \Drupal\core_views_facets\CoreViewsFacetsExposedFilterTypeInterface $facet_core_views_filter_plugin */
      $facet_core_views_filter_plugin = $this->exposedFilterTypePluginManager->createInstance('generic');
    }

    return $facet_core_views_filter_plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $plugin_id_array = explode(':', $this->pluginId);
    list($view_id, ) = explode('__', $plugin_id_array[1]);
    $dependencies['config'] = ['views.view.' . $view_id];
    $dependencies['module'] = ['core_views_facets', 'views'];

    return $dependencies;
  }

}
