<?php

namespace Drupal\core_views_facets\Plugin\facets\facet_source;

use Symfony\Component\Routing\Exception\RouteNotFoundException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\facets\FacetInterface;
use Drupal\facets\FacetSource\FacetSourcePluginBase;
use Drupal\views\Views;

/**
 * Provide common functions for core Views based facet sources.
 */
abstract class CoreViewsFacetSourceBase extends FacetSourcePluginBase {

  /**
   * The current view.
   *
   * @var \Drupal\views\ViewExecutable
   */
  protected $view;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager|null
   */
  protected $entityTypeManager;

  /**
   * The url processor name.
   *
   * @var string
   */
  protected $urlProcessor = 'core_views_url_processor';

  /**
   * Return the current view.
   *
   * @return \Drupal\views\ViewExecutable
   *   Current view.
   */
  public function getView() {
    return $this->view;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, array $configuration, $plugin_id, array $plugin_definition, $query_type_plugin_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $query_type_plugin_manager);

    $this->pluginDefinition = $plugin_definition;
    $this->pluginId = $plugin_id;
    $this->configuration = $configuration;
    $this->entityTypeManager = $entity_type_manager;

    $view = Views::getView($plugin_definition['view_id']);
    if (!empty($view)) {
      $view->setDisplay($plugin_definition['view_display']);
      $this->view = $view;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getPath() {
    $path = $this->view->getPath();
    if ($path[0] !== '/') {
      $path = '/' . $path;
    }
    return $path;
  }

  /**
   * {@inheritdoc}
   */
  public function isRenderedInCurrentRequest() {
    $request = \Drupal::requestStack()->getMasterRequest();
    if ($request->attributes->get('_controller') === 'Drupal\views\Routing\ViewPageController::handle') {
      list(, $view) = explode(':', $this->getPluginId());
      list($view_id, $view_display) = explode('__', $view);

      if ($request->attributes->get('view_id') == $view_id && $request->attributes->get('display_id') == $view_display) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $form['field_identifier'] = [
      '#type' => 'select',
      '#options' => [],
      '#title' => $this->t('Facet field'),
      '#description' => $this->t('Choose the filter to facet by.'),
      '#required' => TRUE,
      '#default_value' => $this->facet->getFieldIdentifier(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $facet_source_id = $this->facet->getFacetSourceId();
    $triggering_element = $form_state->getTriggeringElement();

    if (!empty($triggering_element['#name']) && $triggering_element['#name'] == 'facet_source_configure') {
      return;
    }

    $views_filters = $this->getFields();
    $field_identifier = $form_state->getValue('facet_source_configs')[$facet_source_id]['field_identifier'];

    if (empty($field_identifier) || empty($views_filters[$field_identifier])) {
      $form_state->setErrorByName('facet_source_id', t('Please select a valid field.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getQueryTypesForFacet(FacetInterface $facet) {
    // We don't actually use this. Just put something that exists.
    return [
      'string' => 'search_api_string',
    ];
  }

  /**
   * Retrieves the current views arguments map and returns a detailed version.
   *
   * @return array
   *   Detail enriched argument data.
   */
  public function getViewsArgumentsMap() {
    if (empty($this->view->getDisplay()->getRoutedDisplay())) {
      return [];
    }

    $views_arguments = [];
    foreach ($this->view->getHandlers('argument', $this->view->current_display) as $argument_id => $argument) {
      $views_arguments[$argument_id] = $this->view->display_handler->getHandler('argument', $argument_id);
    }
    reset($views_arguments);

    $route_name = $this->view->getUrl()->getRouteName();
    try {
      /** @var \Symfony\Component\Routing\Route $route */
      $route = \Drupal::service('router.route_provider')->getRouteByName($route_name);
    }
    catch (RouteNotFoundException $e) {
      return [];
    }
    $route_map = $route->hasOption('_view_argument_map') ? $route->getOption('_view_argument_map') : array();

    $route_match = \Drupal::routeMatch();

    // Where do arguments come from?
    if ($this->isRenderedInCurrentRequest()) {
      if ($ajax_arguments = drupal_static('core_views_contextual_filter_ajax_arguments')) {
        $argument_source = 'view_ajax';
        reset($ajax_arguments);
      }
      else {
        $argument_source = 'view_page';
      }
    }
    else {
      $argument_source = 'view_defaults';
    }

    // Each argument can only be active, if each previous argument was active
    // too. This variable is set to FALSE for all following arguments, when any
    // one argument is inactive.
    $active_value = TRUE;

    $map = [];
    // For all I can tell, the association view argument <-> route argument
    // happens entirely by counting up in both arrays...
    foreach ($route_map as $attribute => $parameter_name) {
      $current_map_entry = [];

      // Allow parameters be pulled from the request.
      // The map stores the actual name of the parameter in the request. Views
      // which override existing controller, use for example 'node' instead of
      // arg_nid as name.
      if (isset($map[$attribute])) {
        $attribute = $map[$attribute];
      }

      $current_map_entry['route_parameter'] = $attribute;
      $current_map_entry['views_argument_id'] = key($views_arguments);
      $current_map_entry['views_argument'] = current($views_arguments);
      next($views_arguments);

      $current_map_entry['neutral_value'] = empty($current_map_entry['views_argument']->options['exception']['value']) ? 'all' : $current_map_entry['views_argument']->options['exception']['value'];

      switch ($argument_source) {
        case 'view_page':
          // This is views page request, standard procedure.
          if ($current_request_argument = $route_match->getRawParameter($attribute)) {
          }
          else {
            $current_request_argument = $route_match->getParameter($attribute);
          }

          if (isset($current_request_argument)) {
            $current_map_entry['value'] = $current_request_argument;
            $current_map_entry['active'] = $active_value;
          }
          else {
            $current_map_entry['value'] = $current_map_entry['neutral_value'];
            $current_map_entry['active'] = FALSE;
            $active_value = FALSE;
          }
          break;

        case 'view_ajax':
          if (current($ajax_arguments)) {
            $current_map_entry['value'] = current($ajax_arguments);
            $current_map_entry['active'] = $active_value;
            next($ajax_arguments);
          }
          else {
            $current_map_entry['value'] = $current_map_entry['neutral_value'];
            $current_map_entry['active'] = FALSE;
            $active_value = FALSE;
          }
          break;

        default:
          // Just add the defaults.
          $current_map_entry['value'] = $current_map_entry['neutral_value'];
          $current_map_entry['active'] = FALSE;
          $active_value = FALSE;
      }

      $map[$current_map_entry['views_argument_id']] = $current_map_entry;
    }

    return $map;
  }

  /**
   * Optionally get JS settings when AJAX is enabled.
   *
   * @param \Drupal\facets\FacetInterface $facet
   *   The facet to get settings for.
   *
   * @return array
   *   The drupalSettings array.
   */
  public function getAjaxSettingsByFacet(FacetInterface $facet) {
    /** @var \Drupal\core_views_facets\Plugin\facets\facet_source\CoreViewsFacetSourceBase $facet_source */
    $facet_source = $facet->getFacetSource();

    /** @var \Drupal\views\ViewExecutable $view */
    $view = $facet_source->getView();

    $filter_handler = $view->getHandler($view->current_display, 'filter', $facet->getFieldIdentifier());

    return [
      'view_id' => $view->id(),
      'current_display_id' => $view->current_display,
      'field_id' => $filter_handler['expose']['identifier'],
      'type' => $facet_source->getBaseId(),
      'view_base_path' => $view->getPath(),
    ];
  }

}
